
.nagare-error-field, .nagare-error-input, .nagare-error-message {
    display: inline;
}

.nagare-error-input {
	background: url("redline.png") left bottom repeat-x;
    padding-bottom: 4px;
}

.nagare-error-message {
    color:  #c30;
    font-size: 60%;
    padding-left: 3px;
    font-weight: bold;
}

fieldset {
    border: 1px solid #3f7516;
    margin: 8px 0;
}

input[type="submit"], input[type="button"] {
    color: white;
    border: 0;
    background-color: #3f7516;
}

input[type="submit"]:hover {
    background-color: #52891d;
}